@@ -41,6 +41,7 @@
 //M*/
 
 #include "test_precomp.hpp"
+#include "opencv2/imgproc/imgproc_c.h"
 
 using namespace cv;
 using namespace std;
@@ -74,6 +75,9 @@ CV_DefaultNewCameraMatrixTest::CV_DefaultNewCameraMatrixTest()
     test_array[INPUT].push_back(NULL);
     test_array[OUTPUT].push_back(NULL);
     test_array[REF_OUTPUT].push_back(NULL);
+
+    matrix_type = 0;
+    center_principal_point = false;
 }
 
 void CV_DefaultNewCameraMatrixTest::get_test_array_types_and_sizes( int test_case_idx, vector<vector<Size> >& sizes, vector<vector<int> >& types )
@@ -199,6 +203,9 @@ CV_UndistortPointsTest::CV_UndistortPointsTest()
     test_array[OUTPUT].push_back(NULL); // distorted dst points
     test_array[TEMP].push_back(NULL); // dst points
     test_array[REF_OUTPUT].push_back(NULL);
+
+    useCPlus = useDstMat = false;
+    zero_new_cam = zero_distortion = zero_R = false;
 }
 
 void CV_UndistortPointsTest::get_test_array_types_and_sizes( int test_case_idx, vector<vector<Size> >& sizes, vector<vector<int> >& types )
@@ -491,7 +498,7 @@ void CV_UndistortPointsTest::distortPoints(const CvMat* _src, CvMat* _dst, const
         __P = cvCreateMat(3,4,CV_64F);
     if (matP)
     {
-        cvTsConvert(matP,__P);
+        cvtest::convert(cvarrToMat(matP), cvarrToMat(__P), -1);
     }
     else
     {
@@ -500,7 +507,7 @@ void CV_UndistortPointsTest::distortPoints(const CvMat* _src, CvMat* _dst, const
         __P->data.db[4] = 1;
         __P->data.db[8] = 1;
     }
-    CvMat* __R = cvCreateMat(3,3,CV_64F);;
+    CvMat* __R = cvCreateMat(3,3,CV_64F);
     if (matR)
     {
         cvCopy(matR,__R);
@@ -604,6 +611,11 @@ CV_InitUndistortRectifyMapTest::CV_InitUndistortRectifyMapTest()
     test_array[INPUT].push_back(NULL); // new camera matrix
     test_array[OUTPUT].push_back(NULL); // distorted dst points
     test_array[REF_OUTPUT].push_back(NULL);
+
+    useCPlus = false;
+    zero_distortion = zero_new_cam = zero_R = false;
+    _mapx = _mapy = NULL;
+    mat_type = 0;
 }
 
 void CV_InitUndistortRectifyMapTest::get_test_array_types_and_sizes( int test_case_idx, vector<vector<Size> >& sizes, vector<vector<int> >& types )
@@ -847,16 +859,16 @@ void CV_InitUndistortRectifyMapTest::prepare_to_validation(int/* test_case_idx*/
     CvMat _input3 = test_mat[INPUT][3];
     CvMat _input4 = test_mat[INPUT][4];
 
-    cvTsConvert(&_input1,&_camera);
-    cvTsConvert(&_input2,&_distort);
-    cvTsConvert(&_input3,&_rot);
-    cvTsConvert(&_input4,&_new_cam);
+    cvtest::convert(cvarrToMat(&_input1), cvarrToMat(&_camera), -1);
+    cvtest::convert(cvarrToMat(&_input2), cvarrToMat(&_distort), -1);
+    cvtest::convert(cvarrToMat(&_input3), cvarrToMat(&_rot), -1);
+    cvtest::convert(cvarrToMat(&_input4), cvarrToMat(&_new_cam), -1);
 
     //Applying precalculated undistort rectify map
     if (!useCPlus)
     {
-        mapx = cv::Mat(_mapx);
-        mapy = cv::Mat(_mapy);
+        mapx = cv::cvarrToMat(_mapx);
+        mapy = cv::cvarrToMat(_mapy);
     }
     cv::Mat map1,map2;
     cv::convertMaps(mapx,mapy,map1,map2,CV_32FC1);
@@ -876,7 +888,7 @@ void CV_InitUndistortRectifyMapTest::prepare_to_validation(int/* test_case_idx*/
                       zero_distortion ? 0 : &_distort, zero_R ? 0 : &_rot, zero_new_cam ? &_camera : &_new_cam);
     //cvTsDistortPoints(&_points,&ref_points,&_camera,&_distort,&_rot,&_new_cam);
     CvMat dst = test_mat[REF_OUTPUT][0];
-    cvTsConvert(&ref_points,&dst);
+    cvtest::convert(cvarrToMat(&ref_points), cvarrToMat(&dst), -1);
 
     cvtest::copy(test_mat[INPUT][0],test_mat[OUTPUT][0]);
 
